import java.sql.*;
import java.util.Vector;

class DBLogin
{
	public	String	username;
	public	String	password;
	public	String	connurl;
	
	public DBLogin(String u, String p, String c)
	{
		username = u;
		password = p;
		connurl = c;
	}
}

class DBField
{
	public	int	fnum;
	public	int	ftyp;
	public	int	fsiz;
	public	int	fdig;
	public	String	fnam;
	public	String	fdef;
	
	public DBField(ResultSet res) throws SQLException
	{
		fnum = res.getInt(17);
		ftyp = res.getInt(5);
		fsiz = res.getInt(7);
		fdig = res.getInt(9);
		ftyp = res.getInt(5);
		fnam = res.getString(4).toLowerCase();
		fdef = res.getString(6);
	
		if (fdef.equals("VARCHAR2")) {
			fdef = "VARCHAR";
		}
		
		if (fdef.equals("DATE")) {
			fdef = "DATETIME";
		}
		
		if (fdef.equals("VARCHAR")) {
			fdef = fdef + "(" + fsiz + ")";
		}
		if (fdef.equals("NUMBER")) {
			if (fdig == 0) {
				// integer
				fdef = "int4";
			} else {
				if (fdig == 2) {
					fdef = "money";
				} else {
					fdef = "decimal(" + fsiz + "," + fdig + ")";
				}
			}
		}
		fdef = fdef.toLowerCase();
	}
	
	public String toString()
	{
		return fnam + " " + fdef;
	}
}
			
class DBXfer
{
	Connection	srcConn;
	Connection	dstConn;
	DBLogin		srcLogin;
	DBLogin		dstLogin;

	public static void main(String argv[])
	{
		try {
			Class.forName ("oracle.jdbc.driver.OracleDriver");
			DriverManager.registerDriver (new oracle.jdbc.driver.OracleDriver());
	
			Class.forName ("postgresql.Driver");
			DriverManager.registerDriver (new postgresql.Driver());
		} catch (Exception e) {
			System.out.println("Cound Not Register Drivers");
			return;
		}

		DBLogin	srcLogin = new DBLogin("user","pass","jdbc:oracle:thin:@<host>:1521:ORCL");
		DBLogin	dstLogin = new DBLogin("user","pass","jdbc:postgresql://<host>/<db>");
		DBXfer dbx = new DBXfer(srcLogin,dstLogin);
		
		dbx.transfer(null,"schema","table");
	}
	
	public DBXfer(DBLogin src, DBLogin dst)
	{
		srcLogin = src;
		dstLogin = dst;
		
		try {
			srcConn = DriverManager.getConnection(src.connurl,src.username,src.password);
		} catch (Exception e) {
			System.out.println("Could not open source connection: " + e);
			return;
		}
		try {
			dstConn = DriverManager.getConnection(dst.connurl,dst.username,dst.password);
		} catch (Exception e) {
			System.out.println("Could not open destination connection: " + e);
			srcConn = null;
			return;
		}
	}

	public void transfer(String catalog, String schema, String table)
	{
		ResultSet	res;
		String		srcsql;
		String		dstsql;
		String		pkey[];
		int		pmax;
		DBField		fdat[];
		int		fmax;
		int		i;
		String		com;

		srcsql = "create table " + table.toLowerCase() + " ( ";
		dstsql = "insert into " + table.toLowerCase() + " ( ";
		com = "";
		pmax = 0;
		pkey = new String[16];
		fmax = 0;
		fdat = new DBField[100];

		try {
			DatabaseMetaData dmd = srcConn.getMetaData();
			
			res = dmd.getColumns(catalog,schema,table,"%");
			while (res.next()) {
				DBField	tmp = new DBField(res);
				fdat[tmp.fnum] = tmp;
				if (tmp.fnum > fmax) {
					fmax = tmp.fnum;
				}
			}
			
			for (i=1;i<fmax+1;i++) {
				srcsql = srcsql + com + fdat[i].toString();
				dstsql = dstsql + com + fdat[i].fnam;
				com = ", ";
			}

			res = dmd.getPrimaryKeys(catalog,schema,table);
			while (res.next()) {
				//System.out.println("Primary("+res.getString(5)+"): " + res.getString(4));
				pkey[res.getInt(5)] = res.getString(4);
				if (res.getInt(5) > pmax) {
					pmax = res.getInt(5);
				}
			}
			if (pmax > 0) { // got a primary key
				srcsql = srcsql + com + "primary key ( ";
				com = "";
				for (i=1; i<pmax+1;i++) {
					srcsql = srcsql + com + pkey[i].toLowerCase();;
					com = ", ";
				}
				srcsql = srcsql + " )";
			}

			srcsql = srcsql + " )";
			System.out.println(srcsql);
			
			// finish building destination insert statement
			dstsql = dstsql + " ) values ( ";
			com = "";
			for (i=1;i<fmax+1;i++) {
				dstsql = dstsql + com + "?";
				com = ", ";
			}
			dstsql = dstsql + " )";
			System.out.println(dstsql);
						
			// we got everything we need.... lets do something with it !!
			Statement dstcmd = dstConn.createStatement();
			
			// drop the table if it exists
			try {
				dstcmd.executeUpdate("drop table " + table);
			} catch (Exception e) {}
			
			// create table using the definition we just assembled
			System.out.println("creating table");
			System.out.flush();
			dstcmd.executeUpdate(srcsql);
			
			// we got a table.... get the data
			Statement srccmd = srcConn.createStatement();
			System.out.println("accessing source data");
			System.out.flush();
			res = srccmd.executeQuery("select * from " + table);
			
			// setup to store data to new table
			System.out.println("building insert statement");
			System.out.flush();
			PreparedStatement dstins = dstConn.prepareStatement(dstsql);
			
			// move that data !!
			int cnt = 0;
			String fld;
			while (res.next()) {
				System.out.println("======== Record: " + (++cnt));
				System.out.flush();
				for (i=1;i<fmax+1;i++) {
					fld = res.getString(i);
					if (fld == null) {
						System.out.println(i + "\t" + fdat[i].ftyp + "\t" + fdat[i].fdig + "\t" + null);
					} else {
						System.out.println(i + "\t" + fdat[i].ftyp + "\t" + fdat[i].fdig + "\t" + fld.toString());
					}
					System.out.flush();
					if (fld == null) {
						dstins.setNull(i,fdat[i].ftyp);
					} else {
						dstins.setString(i,fld);
					}
				}
				dstins.executeUpdate();
			}
			
		} catch (Exception e) {
			System.out.println("Exception: " + e);
		}
	}
}